% This program is made by Md. Iftekhar Tanveer (go2chayan@gmail.com)
%
% This matlab code is intended to demonstrate the physical
% significance/interpretation of the elements of diagonal matrix in
% Singular Value Decomposition (SVD).
% 
% Here, multiple rows of a BIG matrix is gradually being filled with 'ones'
% which means the information content is being reduced. Now, the
% eigenvalues and their 1st and 2nd derivatives are plotted.
%
% From the plot is can be seen that the number of eigenvalues with high
% values have a strong relationship with information content in the matrix.
% This measurement is accurate even when the matrix is non-singular. The
% stability of this measurement can be checked by incorporating some noise
% as done in 'option 1' of the code.

n = 400;
a = rand(n);

for i=0:50:399;
    
    % For seeing the effect of noise, comment out the line following
    % option 2 and uncomment the line following option 1
    
    % option 1
    %a(1:i,:)=ones(i,n)+0.05*randn(i,n);
    % option 2
    a(1:i,:)=ones(i,n);
    
    [U,D,V]=svd(a);
    d = diag(D);d=d(2:end);
    Gd1 = gradient(d);
    Gd2 = gradient(Gd1);
    figure
    subplot(221);imagesc(a);
    subplot(222);plot(d);
    subplot(223);plot(Gd1);
    subplot(224);plot(Gd2);
    
    fprintf(1,'trace = %f\n',sum(d));
    fprintf(1,'determinant = %f\n',prod(d));
end